@@ -4,7 +4,7 @@
 # SOAP::Lite is free software; you can redistribute it
 # and/or modify it under the same terms as Perl itself.
 #
-# $Id: Packager.pm,v 1.4 2004/10/26 15:26:49 byrnereese Exp $
+# $Id: Packager.pm 249 2008-05-05 20:35:05Z kutterma $
 #
 # ======================================================================
 
@@ -13,7 +13,8 @@ package SOAP::Packager;
 use strict;
 use vars;
 
-use vars qw($SUPPORTED_TYPES);
+use vars qw($VERSION $SUPPORTED_TYPES);
+use version; $VERSION = qv('0.710.05');
 $SUPPORTED_TYPES = { };
 
 sub BEGIN {
@@ -56,7 +57,7 @@ sub find_packager {
    # TODO - Input:
    #        * the mimetype of the data to be decoded raw data that needs
    #        * the data to be decoded
-   #        Returns: 
+   #        Returns:
    #        * the proper SOAP::Packager instance
 }
 
@@ -131,7 +132,7 @@ sub get_multipart_id {
   my ($id) = shift;
   ($id || '') =~ /^<?([^>]+)>?$/; $1 || '';
 }
- 
+
 sub package {
    my $self = shift;
    my ($envelope,$context) = @_;
@@ -140,7 +141,8 @@ sub package {
    require MIME::Entity;
    local $MIME::Entity::BOUNDARY_DELIMITER = "\r\n";
    my $top = MIME::Entity->build('Type'     => "Multipart/Related");
-   $top->attach('Type'                      => $context->soapversion == 1.1 ? "text/xml" : "application/soap+xml",
+   my $soapversion = defined($context) ? $context->soapversion : '1.1';
+   $top->attach('Type'                      => $soapversion == 1.1 ? "text/xml" : "application/soap+xml",
                 'Content-Transfer-Encoding' => $self->transfer_encoding(),
                 'Content-Location'          => $self->env_location(),
                 'Content-ID'                => $self->env_id(),
@@ -191,12 +193,12 @@ sub unpackage {
   }
 }
 
-sub process_form_data { 
+sub process_form_data {
   my ($self, $entity) = @_;
-  my $env = undef;  
+  my $env = undef;
   foreach my $part ($entity->parts) {
     my $name = $part->head->mime_attr('content-disposition.name');
-    $name eq 'payload' ? 
+    $name eq 'payload' ?
       $env = $part->bodyhandle->as_string
 	: $self->push_part($part);
   }
@@ -208,8 +210,14 @@ sub process_related {
   my ($entity) = @_;
   die "Multipart MIME messages MUST declare Multipart/Related content-type"
     if ($entity->head->mime_attr('content-type') !~ /^multipart\/related/i);
-  my $start = get_multipart_id($entity->head->mime_attr('content-type.start'))
-    || get_multipart_id($entity->parts(0)->head->mime_attr('content-id'));
+  # As it turns out, the Content-ID and start parameters are optional
+  # according to the MIME and SOAP specs. In the event that the head cannot
+  # be found, the head/root entity is used as a starting point.
+  my $start = get_multipart_id($entity->head->mime_attr('content-type.start'));
+  if (!defined($start) || $start eq "") {
+      $start = $self->generate_random_string(10);
+      $entity->parts(0)->head->add('content-id',$start);
+  }
   my $location = $entity->head->mime_attr('content-location') ||
     'thismessage:/';
   my $env;
@@ -237,8 +245,8 @@ sub process_related {
       $self->push_part($part) if (defined($part->bodyhandle));
     }
   }
-  die "Can't find 'start' parameter in multipart MIME message\n"
-    if @{$self->parts} > 1 && !$start;
+#  die "Can't find 'start' parameter in multipart MIME message\n"
+#    if @{$self->parts} > 1 && !$start;
   return $env;
 }
 
@@ -287,7 +295,8 @@ sub package {
    require DIME::Payload;
    my $message = DIME::Message->new;
    my $top = DIME::Payload->new;
-   $top->attach('MIMEType' => $context->soapversion == 1.1 ? 
+   my $soapversion = defined($context) ? $context->soapversion : '1.1';
+   $top->attach('MIMEType' => $soapversion == 1.1 ?
                   "http://schemas.xmlsoap.org/soap/envelope/" : "application/soap+xml",
                 'Data'     => $envelope );
    $message->add_payload($top);
@@ -348,7 +357,7 @@ over the wire.
 
 =head1 METHODS
 
-=over 
+=over
 
 =item new
 
@@ -358,7 +367,7 @@ Instantiates a new instance of a SOAP::Packager.
 
 Contains an array of parts. The contents of this array and their types are completely
 dependant upon the Packager being used. For example, when using MIME, the content
-of this array is MIME::Entity's. 
+of this array is MIME::Entity's.
 
 =item push_part
 
@@ -381,7 +390,7 @@ override the Content-Type (e.g. multipart/related, or application/dime).
 If you wish to implement your own SOAP::Packager, then the methods below must be
 implemented by you according to the prescribed input and output requirements.
 
-=over 
+=over
 
 =item package()
 
@@ -506,12 +515,12 @@ the client.
         Path        => "printenv",
         Filename    => "printenv",
         Disposition => "attachment";
-      # read attachments                                                                                                         
+      # read attachments
       foreach my $part (@{$envelope->parts}) {
         print STDERR "soaplite.cgi: attachment found! (".ref($part).")\n";
         print STDERR "soaplite.cgi: contents => ".$part->stringify."\n";
       }
-      # send attachments                                                                                                         
+      # send attachments
       return SOAP::Data->name('convertedTemp' => (((9/5)*($temp->value)) + 32)),
         $ent;
     }
@@ -523,7 +532,7 @@ TODO
 
 =head1 SEE ALSO
 
-L<MIME::Tools>, L<DIME::Tools> 
+L<MIME::Tools>, L<DIME::Tools>
 
 =head1 COPYRIGHT
 
